@@ -29,7 +29,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <kdeclarative/kdeclarative.h>
 #include <KDeclarative/KQuickAddons/QuickViewSharedEngine>
 #include <KUser>
-#include <Solid/PowerManagement>
 //Plasma
 #include <KPackage/Package>
 #include <KPackage/PackageStructure>
@@ -139,9 +138,9 @@ void UnlockApp::initialize()
     KScreenSaverSettings::self()->load();
     KPackage::Package package = KPackage::PackageLoader::self()->loadPackage(QStringLiteral("Plasma/LookAndFeel"));
     KConfigGroup cg(KSharedConfig::openConfig(QStringLiteral("kdeglobals")), "KDE");
-    const QString packageName = cg.readEntry("LookAndFeelPackage", QString());
-    if (!packageName.isEmpty()) {
-        package.setPath(packageName);
+    m_packageName = cg.readEntry("LookAndFeelPackage", QString());
+    if (!m_packageName.isEmpty()) {
+        package.setPath(m_packageName);
     }
     if (!KScreenSaverSettings::theme().isEmpty()) {
         package.setPath(KScreenSaverSettings::theme());
@@ -182,8 +181,6 @@ void UnlockApp::desktopResized()
     }
 
     // extend views and savers to current demand
-    const bool canLogout = KAuthorized::authorizeKAction(QStringLiteral("logout")) && KAuthorized::authorize(QStringLiteral("logout"));
-    const QSet<Solid::PowerManagement::SleepState> spdMethods = Solid::PowerManagement::supportedSleepStates();
     for (int i = m_views.count(); i < nScreens; ++i) {
         connect(QGuiApplication::screens()[i], &QObject::destroyed, this, &UnlockApp::desktopResized);
         // create the view
@@ -232,7 +229,8 @@ void UnlockApp::desktopResized()
         context->setContextProperty(QStringLiteral("kscreenlocker_userImage"), user.faceIconPath());
         context->setContextProperty(QStringLiteral("authenticator"), m_authenticator);
         context->setContextProperty(QStringLiteral("backgroundPath"), KScreenSaverSettings::themeBackground());
-        context->setContextProperty(QStringLiteral("org_kde_plasma_screenlocker_greeter_interfaceVersion"), 1);
+        context->setContextProperty(QStringLiteral("org_kde_plasma_screenlocker_greeter_interfaceVersion"), 2);
+        context->setContextProperty(QStringLiteral("org_kde_plasma_screenlocker_greeter_view"), view);
 
         view->setSource(m_mainQmlPath);
         // on error, load the fallback lockscreen to not lock the user out of the system
@@ -249,16 +247,14 @@ void UnlockApp::desktopResized()
         lockProperty.write(m_immediateLock || (!m_noLock && !m_delayedLockTimer));
 
         QQmlProperty sleepProperty(view->rootObject(), QStringLiteral("suspendToRamSupported"));
-        sleepProperty.write(spdMethods.contains(Solid::PowerManagement::SuspendState));
-        if (spdMethods.contains(Solid::PowerManagement::SuspendState) &&
-            view->rootObject()->metaObject()->indexOfSignal(QMetaObject::normalizedSignature("suspendToRam()").constData()) != -1) {
+        sleepProperty.write(m_canSuspend);
+        if (view->rootObject()->metaObject()->indexOfSignal(QMetaObject::normalizedSignature("suspendToRam()").constData()) != -1) {
             connect(view->rootObject(), SIGNAL(suspendToRam()), SLOT(suspendToRam()));
         }
 
         QQmlProperty hibernateProperty(view->rootObject(), QStringLiteral("suspendToDiskSupported"));
-        hibernateProperty.write(spdMethods.contains(Solid::PowerManagement::HibernateState));
-        if (spdMethods.contains(Solid::PowerManagement::HibernateState) &&
-            view->rootObject()->metaObject()->indexOfSignal(QMetaObject::normalizedSignature("suspendToDisk()").constData()) != -1) {
+        hibernateProperty.write(m_canHibernate);
+        if (view->rootObject()->metaObject()->indexOfSignal(QMetaObject::normalizedSignature("suspendToDisk()").constData()) != -1) {
             connect(view->rootObject(), SIGNAL(suspendToDisk()), SLOT(suspendToDisk()));
         }
 
@@ -385,7 +381,7 @@ void UnlockApp::suspendToRam()
     m_ignoreRequests = true;
     m_resetRequestIgnoreTimer->start();
 
-    Solid::PowerManagement::requestSleep(Solid::PowerManagement::SuspendState, 0, 0);
+    org_kde_ksld_suspendSystem(m_ksldInterface);
 
 }
 
@@ -398,7 +394,7 @@ void UnlockApp::suspendToDisk()
     m_ignoreRequests = true;
     m_resetRequestIgnoreTimer->start();
 
-    Solid::PowerManagement::requestSleep(Solid::PowerManagement::HibernateState, 0, 0);
+    org_kde_ksld_hibernateSystem(m_ksldInterface);
 }
 
 void UnlockApp::setTesting(bool enable)
@@ -419,6 +415,19 @@ void UnlockApp::setTesting(bool enable)
     }
 }
 
+void UnlockApp::setTheme(const QString &theme)
+{
+    if (theme.isEmpty() || !m_testing) {
+        return;
+    }
+
+    m_packageName = theme;
+    KPackage::Package package = KPackage::PackageLoader::self()->loadPackage(QStringLiteral("Plasma/LookAndFeel"));
+    package.setPath(m_packageName);
+
+    m_mainQmlPath = QUrl::fromLocalFile(package.filePath("lockscreenmainscript"));
+}
+
 void UnlockApp::setImmediateLock(bool immediate)
 {
     m_immediateLock = immediate;
@@ -522,9 +531,23 @@ static void osdText(void *data, org_kde_ksld *org_kde_ksld, const char *icon, co
     reinterpret_cast<UnlockApp*>(data)->osdText(QString::fromUtf8(icon), QString::fromUtf8(text));
 }
 
+static void canSuspend(void *data, org_kde_ksld *org_kde_ksld, uint suspend)
+{
+    Q_UNUSED(org_kde_ksld)
+    reinterpret_cast<UnlockApp*>(data)->updateCanSuspend(suspend);
+}
+
+static void canHibernate(void *data, org_kde_ksld *org_kde_ksld, uint hibernate)
+{
+    Q_UNUSED(org_kde_ksld)
+    reinterpret_cast<UnlockApp*>(data)->updateCanHibernate(hibernate);
+}
+
 static const struct org_kde_ksld_listener s_listener {
     osdProgress,
-    osdText
+    osdText,
+    canSuspend,
+    canHibernate
 };
 
 void UnlockApp::setKsldSocket(int socket)
@@ -597,5 +620,29 @@ void UnlockApp::osdText(const QString &icon, const QString &additionalText)
     }
 }
 
+void UnlockApp::updateCanSuspend(bool set)
+{
+    if (m_canSuspend == set) {
+        return;
+    }
+    m_canSuspend = set;
+    for (auto it = m_views.constBegin(), end = m_views.constEnd(); it != end; ++it) {
+        QQmlProperty sleepProperty((*it)->rootObject(), QStringLiteral("suspendToRamSupported"));
+        sleepProperty.write(m_canSuspend);
+    }
+}
+
+void UnlockApp::updateCanHibernate(bool set)
+{
+    if (m_canHibernate == set) {
+        return;
+    }
+    m_canHibernate = set;
+    for (auto it = m_views.constBegin(), end = m_views.constEnd(); it != end; ++it) {
+        QQmlProperty hibernateProperty((*it)->rootObject(), QStringLiteral("suspendToDiskSupported"));
+        hibernateProperty.write(m_canHibernate);
+    }
+}
+
 } // namespace
 
